@@ -1,7 +1,7 @@
 # Orca
 #
 # Copyright 2004-2009 Sun Microsystems Inc.
-# Copyright 2010 Joanmarie Diggs, Mesar Hameed
+# Copyright 2010-2011 The Orca Team
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -24,11 +24,13 @@ __id__        = "$Id$"
 __version__   = "$Revision$"
 __date__      = "$Date$"
 __copyright__ = "Copyright (c) 2004-2009 Sun Microsystems Inc." \
-                "Copyright (c) 2010 Joanmarie Diggs, Mesar Hameed."
+                "Copyright (c) 2010-2011 The Orca Team"
 __license__   = "LGPL"
 
 import getopt
 import os
+import subprocess
+import re
 import signal
 import sys
 import time
@@ -74,15 +76,6 @@ def onEnabledChanged(gsetting, key):
 
     if key == 'screen-reader-enabled' and not enabled:
         shutdown()
-    elif key == 'screen-magnifier-enabled':
-        settings.enableMagnifier = enabled
-        if enabled:
-            try:
-                mag.init()
-            except:
-                debug.printException(debug.LEVEL_SEVERE)
-                debug.println(debug.LEVEL_SEVERE,
-                              "Could not initialize connection to magnifier.")
 
 class Options:
     """Class to handle getting run-time options."""
@@ -104,7 +97,6 @@ class Options:
         self.canEnable = {'speech':'enableSpeech',
                           'braille':'enableBraille',
                           'braille-monitor':'enableBrailleMonitor',
-                          'magnifier':'enableMagnifier',
                           'main-window':'showMainWindow',
                           'splash-window':'showSplashWindow'}
 
@@ -353,13 +345,13 @@ class Options:
         returns False."""
 
         if self.cannotEnable:
-            valid = "\nspeech, braille, braille-monitor, magnifier, " \
+            valid = "\nspeech, braille, braille-monitor, " \
                     "main-window, splash-window"
             # Translators: This message is displayed when the user
             # tries to enable or disable a feature via an argument,
             # but specified an invalid feature. Valid features are:
-            # speech, braille, braille-monitor, magnifier, main-window,
-            # and splash-window. These items are not localized and are
+            # speech, braille, braille-monitor, main-window, and
+            # splash-window. These items are not localized and are
             # presented in a list after this message.
             #
             msg = _("The following items can be enabled or disabled:")
@@ -445,7 +437,6 @@ class Options:
                         + "speech" + "|" \
                         + "braille" + "|" \
                         + "braille-monitor" + "|" \
-                        + "magnifier" + "|" \
                         + "main-window" + "|" \
                         + "splash-window" + "] ")
 
@@ -459,7 +450,6 @@ class Options:
                         + "speech" + "|" \
                         + "braille" + "|" \
                         + "braille-monitor" + "|" \
-                        + "magnifier" + "|" \
                         + "main-window" + "|" \
                         + "splash-window" + "] ")
 
@@ -548,12 +538,6 @@ if settings.useDBus:
     import dbus.mainloop.glib
     dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
     import dbusserver
-    try:
-        import gsmag as mag
-    except:
-        import mag
-else:
-    import mag
 
 import braille
 import httpserver
@@ -842,7 +826,15 @@ def isModifierKey(event_string):
                      'Shift_L', 'Shift_R', 'Meta_L', 'Meta_R' ]
 
     if not orca_state.bypassNextCommand:
-        modifierKeys.extend(settings.orcaModifierKeys)
+        orcaModifiers = settings.orcaModifierKeys
+        try:
+            orcaModifiers = map(lambda x: x.encode('UTF-8'), orcaModifiers)
+        except (UnicodeDecodeError, UnicodeEncodeError):
+            pass
+        modifierKeys.extend(orcaModifiers)
+
+    if isinstance(event_string, unicode):
+        event_string.encode('UTF-8')
 
     reply = event_string in modifierKeys
     debug.println(debug.LEVEL_FINEST,
@@ -1053,16 +1045,10 @@ def keyEcho(event):
                                      + " " + _("on")
 
             elif event_string == "Num_Lock":
-                # [[[TODO: richb - we are not getting a correct modifier
-                # state value returned when Num Lock is turned off.
-                # Commenting out the speaking of the bogus on/off state
-                # until this can be fixed.]]]
-                #
-                #if modifiers & (1 << pyatspi.MODIFIER_NUMLOCK):
-                #    eventType = KeyEventType.LOCKING_UNLOCKED
-                #else:
-                #    eventType = KeyEventType.LOCKING_LOCKED
-                pass
+                if modifiers & (1 << pyatspi.MODIFIER_NUMLOCK):
+                    eventType = KeyEventType.LOCKING_UNLOCKED
+                else:
+                    eventType = KeyEventType.LOCKING_LOCKED
 
             if brailleMessage:
                 braille.displayMessage(brailleMessage,
@@ -1159,8 +1145,14 @@ def _processKeyCaptured(event):
             # character. (i.e. "1" instead of "!")
             #
             keymap = Gdk.Keymap.get_default()
-            success, entries = keymap.get_entries_for_keycode(event.hw_code)
-            event.event_string = Gdk.keyval_name(entries[0].keycode)
+            entries_for_keycode = keymap.get_entries_for_keycode(event.hw_code)
+            entries = entries_for_keycode[-1]
+            event.event_string = Gdk.keyval_name(entries[0])
+
+            if not event.event_string:
+                orca_state.capturingKeys = False
+                return False
+
             if event.event_string.startswith("KP") and \
                event.event_string != "KP_Enter":
                 name = Gdk.keyval_name(entries[1].keycode)
@@ -1254,6 +1246,12 @@ def _processKeyboardEvent(event):
     else:
         allPossibleKeysyms = settings.orcaModifierKeys
 
+    try:
+        allPossibleKeysyms = \
+            map(lambda x: x.encode('UTF-8'), allPossibleKeysyms)
+    except (UnicodeDecodeError, UnicodeEncodeError):
+        pass
+
     isOrcaModifier = allPossibleKeysyms.count(keyboardEvent.event_string) > 0
 
     if event.type == pyatspi.KEY_PRESSED_EVENT:
@@ -1417,6 +1415,44 @@ def toggleSilenceSpeech(script=None, inputEvent=None):
         settings.silenceSpeech = True
     return True
 
+def _setXmodmap(xkbmap):
+    """Set the keyboard map using xkbcomp."""
+    p = subprocess.Popen(['xkbcomp', '-w0', '-', os.environ['DISPLAY']],
+        stdin=subprocess.PIPE, stdout=None, stderr=None)
+    p.communicate(xkbmap)
+
+def _setCapsLockAsOrcaModifier(enable):
+    """Enable or disable use of the caps lock key as an Orca modifier key."""
+    interpretCapsLineProg = re.compile(
+        r'^\s*interpret\s+Caps[_+]Lock[_+]AnyOfOrNone\s*\(all\)\s*{\s*$', re.I)
+    capsModLineProg = re.compile(
+        r'^\s*action\s*=\s*SetMods\s*\(\s*modifiers\s*=\s*Lock\s*,\s*clearLocks\s*\)\s*;\s*$', re.I)
+    normalCapsLineProg = re.compile(
+        r'^\s*action\s*=\s*LockMods\s*\(\s*modifiers\s*=\s*Lock\s*\)\s*;\s*$', re.I)
+    normalCapsLine = '        action= LockMods(modifiers=Lock);'
+    capsModLine =    '        action= SetMods(modifiers=Lock,clearLocks);'
+    lines = _originalXmodmap.split('\n')
+    foundCapsInterpretSection = False
+    for i in range(len(lines)):
+        line = lines[i]
+        if not foundCapsInterpretSection:
+            if interpretCapsLineProg.match(line):
+                foundCapsInterpretSection = True
+        else:
+            if enable:
+                if normalCapsLineProg.match(line):
+                    lines[i] = capsModLine
+                    _setXmodmap('\n'.join(lines))
+                    return
+            else:
+                if capsModLineProg.match(line):
+                    lines[i] = normalCapsLine
+                    _setXmodmap('\n'.join(lines))
+                    return
+            if line.find('}'):
+                # Failed to find the line we need to change
+                return
+
 def _createOrcaXmodmap():
     """Makes an Orca-specific Xmodmap so that the keys behave as we
     need them to do. This is especially the case for the Orca modifier.
@@ -1426,21 +1462,12 @@ def _createOrcaXmodmap():
 
     cmd = []
     if "Caps_Lock" in settings.orcaModifierKeys:
-        cmd.append("clear Lock")
+        _setCapsLockAsOrcaModifier(True)
         _capsLockCleared = True
     elif _capsLockCleared:
-        cmd.append("add Lock = Caps_Lock")
+        _setCapsLockAsOrcaModifier(False)
         _capsLockCleared = False
 
-    # Clear other keysyms so that we always treat the Orca modifier as
-    # the Orca modifier (e.g. remove KP_0 from KP_Insert).
-    #
-    for keyName in settings.orcaModifierKeys:
-        if keyName in ["Caps_Lock", "KP_Insert", "Insert"]:
-            cmd.append("keysym %s = %s" % (keyName, keyName))
-
-    os.system("echo '%s' | xmodmap - > /dev/null 2>&1" % "\n".join(cmd))
-
 def _storeXmodmap(keyList):
     """Save the original xmodmap for the keys in keyList before we alter it.
 
@@ -1449,12 +1476,7 @@ def _storeXmodmap(keyList):
     """
 
     global _originalXmodmap
-
-    items = "|".join(keyList)
-    cmd = "xmodmap -pke | grep -E '(%s)'" % items
-    filehandle = os.popen(cmd)
-    _originalXmodmap = filehandle.read()
-    filehandle.close()
+    _originalXmodmap = subprocess.check_output(['xkbcomp', os.environ['DISPLAY'], '-'])
 
 def _restoreXmodmap(keyList=[]):
     """Restore the original xmodmap values for the keys in keyList.
@@ -1464,21 +1486,11 @@ def _restoreXmodmap(keyList=[]):
       to restore the entire saved xmodmap.
     """
 
-    toRestore = []
-    lines = _originalXmodmap.split("\n")
-    if not keyList:
-        toRestore = lines
-
-    for key in keyList:
-        line = filter(lambda k: " %s" % key in k, lines)
-        toRestore.extend(line)
-
     global _capsLockCleared
-    if _capsLockCleared:
-        toRestore.append("add Lock = Caps_Lock")
-        _capsLockCleared = False
-
-    os.system("echo '%s' | xmodmap - > /dev/null 2>&1" % "\n".join(toRestore))
+    _capsLockCleared = False
+    p = subprocess.Popen(['xkbcomp', '-w0', '-', os.environ['DISPLAY']],
+        stdin=subprocess.PIPE, stdout=None, stderr=None)
+    p.communicate(_originalXmodmap)
 
 def loadUserSettings(script=None, inputEvent=None, skipReloadMessage=False):
     """Loads (and reloads) the user settings module, reinitializing
@@ -1501,7 +1513,6 @@ def loadUserSettings(script=None, inputEvent=None, skipReloadMessage=False):
     httpserver.shutdown()
     speech.shutdown()
     braille.shutdown()
-    mag.shutdown()
 
     _scriptManager.deactivate()
 
@@ -1562,19 +1573,6 @@ def loadUserSettings(script=None, inputEvent=None, skipReloadMessage=False):
             debug.println(debug.LEVEL_WARNING,
                           "Could not initialize connection to braille.")
 
-    if a11yAppSettings.get_boolean('screen-magnifier-enabled'):
-        try:
-            mag.init()
-            debug.println(debug.LEVEL_CONFIGURATION,
-                          "Magnification module has been initialized.")
-        except:
-            debug.printException(debug.LEVEL_SEVERE)
-            debug.println(debug.LEVEL_SEVERE,
-                          "Could not initialize connection to magnifier.")
-    else:
-        debug.println(debug.LEVEL_CONFIGURATION,
-                      "Magnification module has NOT been initialized.")
-
     # I'm not sure where else this should go. But it doesn't really look
     # right here.
     try:
@@ -1972,9 +1970,9 @@ def showSplashGUI(script=None, inputEvent=None):
 _initialized = False
 
 def init(registry):
-    """Initialize the orca module, which initializes speech, braille,
-    and mag modules.  Also builds up the application list, registers
-    for AT-SPI events, and creates scripts for all known applications.
+    """Initialize the orca module, which initializes the speech and braille
+    modules.  Also builds up the application list, registers for AT-SPI events,
+    and creates scripts for all known applications.
 
     Returns True if the initialization procedure has run, or False if this
     module has already been initialized.
@@ -2106,8 +2104,6 @@ def shutdown(script=None, inputEvent=None):
         speech.shutdown()
     if settings.enableBraille:
         braille.shutdown()
-    if settings.enableMagnifier:
-        mag.shutdown()
 
     if settings.timeoutCallback and (settings.timeoutTime > 0):
         signal.alarm(0)
