@@ -45,6 +45,7 @@ import braille_generator
 import debug
 import flat_review
 import formatting
+import label_inference
 import keybindings
 import orca_state
 import script_utilities
@@ -83,6 +84,7 @@ class Script:
         self.presentIfInactive = True
 
         self.utilities = self.getUtilities()
+        self.labelInference = self.getLabelInference()
         self.structuralNavigation = self.getStructuralNavigation()
         self.chat = self.getChat()
         self.inputEventHandlers = {}
@@ -211,6 +213,10 @@ class Script:
         """
         return script_utilities.Utilities(self)
 
+    def getLabelInference(self):
+        """Returns the label inference functionality for this script."""
+        return label_inference.LabelInference(self)
+
     def getEnabledStructuralNavigationTypes(self):
         """Returns a list of the structural navigation object types
         enabled in this script.
@@ -262,7 +268,7 @@ class Script:
             return self.bookmarks
 
     def getAppPreferencesGUI(self):
-        """Return a GtkVBox contain the application unique configuration
+        """Return a GtkGrid containing the application unique configuration
         GUI items for the current application.
         """
         return None
@@ -446,8 +452,8 @@ class Script:
         """
 
         try:
-            import gtk.gdk as gdk
-            keyboardEvent.keyval_name = gdk.keyval_name(keyboardEvent.id)
+            from gi.repository import Gdk
+            keyboardEvent.keyval_name = Gdk.keyval_name(keyboardEvent.id)
         except:
             debug.println(debug.LEVEL_FINE,
                           "Could not obtain keyval_name for id: %d" \
