@@ -1,8 +1,9 @@
 # Orca
 #
 # Copyright (C) 2010-2011 The Orca Team
+# Copyright (C) 2011 Igalia, S.L.
 #
-# Author: Joanmarie Diggs <joanmarie.diggs@gmail.com>
+# Author: Joanmarie Diggs <jdiggs@igalia.com>
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -22,7 +23,8 @@
 __id__        = "$Id$"
 __version__   = "$Revision$"
 __date__      = "$Date$"
-__copyright__ = "Copyright (c) 2010-2011 The Orca Team"
+__copyright__ = "Copyright (C) 2010-2011 The Orca Team" \
+                "Copyright (C) 2011 Igalia, S.L."
 __license__   = "LGPL"
 
 import pyatspi
@@ -55,7 +57,7 @@ class Script(default.Script):
 
     CARET_NAVIGATION_KEYS = ['Left', 'Right', 'Up', 'Down', 'Home', 'End']
 
-    def __init__(self, app, isBrowser=False):
+    def __init__(self, app, isBrowser=True):
         """Creates a new script for WebKitGtk applications.
 
         Arguments:
@@ -175,6 +177,33 @@ class Script(default.Script):
         types = self.getEnabledStructuralNavigationTypes()
         return StructuralNavigation(self, types, True)
 
+    def getEnabledStructuralNavigationTypes(self):
+        """Returns a list of the structural navigation object types
+        enabled in this script."""
+
+        enabledTypes = [StructuralNavigation.ANCHOR,
+                        StructuralNavigation.BLOCKQUOTE,
+                        StructuralNavigation.BUTTON,
+                        StructuralNavigation.CHECK_BOX,
+                        StructuralNavigation.CHUNK,
+                        StructuralNavigation.COMBO_BOX,
+                        StructuralNavigation.ENTRY,
+                        StructuralNavigation.FORM_FIELD,
+                        StructuralNavigation.HEADING,
+                        StructuralNavigation.LANDMARK,
+                        StructuralNavigation.LIST,
+                        StructuralNavigation.LIST_ITEM,
+                        StructuralNavigation.LIVE_REGION,
+                        StructuralNavigation.PARAGRAPH,
+                        StructuralNavigation.RADIO_BUTTON,
+                        StructuralNavigation.SEPARATOR,
+                        StructuralNavigation.TABLE,
+                        StructuralNavigation.TABLE_CELL,
+                        StructuralNavigation.UNVISITED_LINK,
+                        StructuralNavigation.VISITED_LINK]
+
+        return enabledTypes
+
     def getUtilities(self):
         """Returns the utilites for this script."""
 
@@ -198,8 +227,7 @@ class Script(default.Script):
             self.updateBraille(event.source)
             return
 
-        if self.utilities.isWebKitGtk(orca_state.locusOfFocus):
-            orca.setLocusOfFocus(event, event.source, False)
+        orca.setLocusOfFocus(event, event.source, False)
 
         default.Script.onCaretMoved(self, event)
 
@@ -248,18 +276,26 @@ class Script(default.Script):
         if role == pyatspi.ROLE_LIST_ITEM and obj.childCount:
             return
 
+        widgetRoles = [pyatspi.ROLE_MENU,
+                       pyatspi.ROLE_MENU_ITEM,
+                       pyatspi.ROLE_LIST_ITEM,
+                       pyatspi.ROLE_RADIO_BUTTON]
+        if role in widgetRoles:
+            default.Script.onFocus(self, event)
+            return
+
         textRoles = [pyatspi.ROLE_HEADING,
                      pyatspi.ROLE_PANEL,
                      pyatspi.ROLE_PARAGRAPH,
-                     pyatspi.ROLE_SECTION]
+                     pyatspi.ROLE_SECTION,
+                     pyatspi.ROLE_TABLE_CELL]
         if role in textRoles:
             return
 
-        if role == pyatspi.ROLE_LINK and obj.childCount:
-            try:
-                text = obj.queryText()
-            except NotImplementedError:
-                orca.setLocusOfFocus(event, obj[0])
+        if not (role == pyatspi.ROLE_LINK and obj.childCount):
+            lastKey, mods = self.utilities.lastKeyAndModifiers()
+            if lastKey in self.CARET_NAVIGATION_KEYS:
+                return
 
         default.Script.onFocus(self, event)
 
@@ -314,30 +350,75 @@ class Script(default.Script):
         """Speak the character at the caret.
 
         Arguments:
-        - obj: an Accessible object that implements the AccessibleText
-          interface
+        - obj: an Accessible object that implements the AccessibleText interface
         """
 
-        if obj.getRole() == pyatspi.ROLE_SEPARATOR:
-            speech.speak(self.speechGenerator.generateSpeech(obj))
-            return
+        boundary = pyatspi.TEXT_BOUNDARY_CHAR
+        objects = self.utilities.getObjectsFromEOCs(obj, boundary)
+        for (obj, start, end, string) in objects:
+            if string:
+                speech.speakCharacter(string)
+            else:
+                speech.speak(self.speechGenerator.generateSpeech(obj))
 
-        default.Script.sayCharacter(self, obj)
-
-    def sayLine(self, obj):
-        """Speaks the line of an AccessibleText object that contains the
-        caret.
+    def sayWord(self, obj):
+        """Speaks the word at the caret.
 
         Arguments:
-        - obj: an Accessible object that implements the AccessibleText
-               interface
+        - obj: an Accessible object that implements the AccessibleText interface
         """
 
-        default.Script.sayLine(self, obj)
+        boundary = pyatspi.TEXT_BOUNDARY_WORD_START
+        objects = self.utilities.getObjectsFromEOCs(obj, boundary)
+        for (obj, start, end, string) in objects:
+            self.sayPhrase(obj, start, end)
 
-        rolesToSpeak = [pyatspi.ROLE_HEADING]
-        if obj.getRole() in rolesToSpeak:
-            speech.speak(self.speechGenerator.getRoleName(obj))
+    def sayLine(self, obj):
+        """Speaks the line at the caret.
+
+        Arguments:
+        - obj: an Accessible object that implements the AccessibleText interface
+        """
+
+        boundary = pyatspi.TEXT_BOUNDARY_LINE_START
+        objects = self.utilities.getObjectsFromEOCs(obj, boundary)
+        for (obj, start, end, string) in objects:
+            self.sayPhrase(obj, start, end)
+
+            # TODO: Move these next items into the speech generator.
+            if obj.getRole() == pyatspi.ROLE_PANEL \
+               and obj.getIndexInParent() == 0:
+                obj = obj.parent
+
+            rolesToSpeak = [pyatspi.ROLE_HEADING, pyatspi.ROLE_LINK]
+            if obj.getRole() in rolesToSpeak:
+                speech.speak(self.speechGenerator.getRoleName(obj))
+
+    def sayPhrase(self, obj, startOffset, endOffset):
+        """Speaks the text of an Accessible object between the given offsets.
+
+        Arguments:
+        - obj: an Accessible object that implements the AccessibleText interface
+        - startOffset: the start text offset.
+        - endOffset: the end text offset.
+        """
+
+        phrase = self.utilities.substring(obj, startOffset, endOffset)
+        if len(phrase) and phrase != "\n":
+            if phrase.decode("UTF-8").isupper():
+                voice = self.voices[settings.UPPERCASE_VOICE]
+            else:
+                voice = self.voices[settings.DEFAULT_VOICE]
+
+            phrase = self.utilities.adjustForRepeats(phrase)
+            links = filter(lambda x: x.getRole() == pyatspi.ROLE_LINK, obj)
+            if links:
+                phrase = self.utilities.adjustForLinks(obj, phrase, startOffset)
+            speech.speak(phrase, voice)
+        else:
+            # Speak blank line if appropriate.
+            #
+            self.sayCharacter(obj)
 
     def skipObjectEvent(self, event):
         """Gives us, and scripts, the ability to decide an event isn't
@@ -354,10 +435,6 @@ class Script(default.Script):
             if event.source.getRole() == pyatspi.ROLE_LINK:
                 return False
 
-            lastKey, mods = self.utilities.lastKeyAndModifiers()
-            if lastKey in self.CARET_NAVIGATION_KEYS:
-                return True
-
         return default.Script.skipObjectEvent(self, event)
 
     def useStructuralNavigationModel(self):
@@ -515,3 +592,27 @@ class Script(default.Script):
                 return
 
         text.setCaretOffset(offset)
+
+    def getTextLineAtCaret(self, obj, offset=None):
+        """Gets the line of text where the caret is.
+
+        Argument:
+        - obj: an Accessible object that implements the AccessibleText
+          interface
+        - offset: an optional caret offset to use. (Not used here at the
+          moment, but needed in the Gecko script)
+
+        Returns the [string, caretOffset, startOffset] for the line of text
+        where the caret is.
+        """
+
+        # TODO - JD: Piece together the full line for braille
+
+        textLine = default.Script.getTextLineAtCaret(self, obj, offset)
+        string = textLine[0].decode('UTF-8')
+        if string.find(self.EMBEDDED_OBJECT_CHARACTER) == -1:
+            return textLine
+
+        textLine[0] = self.utilities.displayedText(obj)
+
+        return textLine
